@@ -1,27 +1,25 @@
 import contextlib
+from functools import lru_cache
 import json
 import os
 import os.path as op
+from pathlib import Path
 import random
 import sys
 import time
 
-from functools import lru_cache
-from pathlib import Path
-
-from keyring.core import get_keyring, load_config, load_env
+import click
+import girder_client as gcl
 from keyring.backend import get_all_keyring
+from keyring.core import get_keyring, load_config, load_env
 from keyring.errors import KeyringError
 from keyring.util.platform_ import config_root
 from keyrings.alt.file import EncryptedKeyring
 
-import girder_client as gcl
-
-from . import get_logger
+from .consts import MAX_CHUNK_SIZE, known_instances_rev
 from .exceptions import LockingError
-from .support.ui import askyesno
-from .utils import ensure_datetime, flattened, flatten, remap_dict
-from .consts import known_instances_rev, MAX_CHUNK_SIZE
+from . import get_logger
+from .utils import ensure_datetime, flatten, flattened, remap_dict, try_multiple
 
 lgr = get_logger()
 
@@ -142,7 +140,7 @@ class GirderCli(gcl.GirderClient):
 
     def dandi_authenticate(self):
         # Shortcut for advanced folks
-        api_key = os.environ.get("DANDI_API_KEY", None)
+        api_key = os.environ.get("DANDI_GIRDER_API_KEY", None)
         if api_key:
             self.authenticate(apiKey=api_key)
             lgr.debug("Successfully authenticated using the key from the envvar")
@@ -279,6 +277,8 @@ class GirderCli(gcl.GirderClient):
 
     def _traverse_asset_girder(self, g, parent_path=None, recursive=True):
         """Helper which operates on girder record"""
+        import requests
+
         a = self._adapt_record(g)
         if parent_path:
             a["path"] = op.join(parent_path, a["name"])
@@ -290,7 +290,9 @@ class GirderCli(gcl.GirderClient):
             # item should be the one we care about
             pass
         elif a["type"] == "item":
-            file_recs = list(self.listFile(g["_id"]))
+            file_recs = try_multiple(
+                5, requests.ConnectionError, 1.1, lambda: list(self.listFile(g["_id"]))
+            )
             if len(file_recs) > 1:
                 lgr.warning("Multiple files found for %s; using oldest one", a["path"])
                 file_recs = [
@@ -371,9 +373,6 @@ class GirderCli(gcl.GirderClient):
                     break
 
     def get_download_file_iter(self, file_id, chunk_size=MAX_CHUNK_SIZE):
-        """
-        """
-
         def downloader(start_at=0):
             # TODO: make it a common decorator here?
             # Will do 3 attempts to avoid some problems due to flaky/overloaded
@@ -556,45 +555,109 @@ class GirderCli(gcl.GirderClient):
                         f"Failed to unlock dandiset {dandiset_identifier} due to: {msg}"
                     )
 
+    NGINX_MAX_CHUNK_SIZE = 400 * (1 << 20)  # 400 MiB
+
+    def _uploadContents(self, uploadObj, stream, size, progressCallback=None):
+        """
+        Uploads contents of a file.  Overridden so that the chunk size can be
+        set on a per-file basis.
+
+        :param uploadObj: The upload object contain the upload id.
+        :type uploadObj: dict
+        :param stream: Readable stream object.
+        :type stream: file-like
+        :param size: The length of the file. This must be exactly equal to the
+            total number of bytes that will be read from ``stream``, otherwise
+            the upload will fail.
+        :type size: str
+        :param progressCallback: If passed, will be called after each chunk
+            with progress information. It passes a single positional argument
+            to the callable which is a dict of information about progress.
+        :type progressCallback: callable
+        """
+        offset = 0
+        uploadId = uploadObj["_id"]
+
+        chunk_size = max(self.MAX_CHUNK_SIZE, (size + 999) // 1000)
+        if chunk_size > self.NGINX_MAX_CHUNK_SIZE:
+            raise Exception("File requires too many chunks to upload")
+
+        with self.progressReporterCls(
+            label=uploadObj.get("name", ""), length=size
+        ) as reporter:
+
+            while True:
+                chunk = stream.read(min(chunk_size, (size - offset)))
+
+                if not chunk:
+                    break
+
+                if isinstance(chunk, str):
+                    chunk = chunk.encode("utf8")
+
+                uploadObj = self.post(
+                    "file/chunk?offset=%d&uploadId=%s" % (offset, uploadId),
+                    data=gcl._ProgressBytesIO(chunk, reporter=reporter),
+                )
+
+                if "_id" not in uploadObj:
+                    raise Exception(
+                        "After uploading a file chunk, did not receive object with _id. "
+                        "Got instead: " + json.dumps(uploadObj)
+                    )
+
+                offset += len(chunk)
+
+                if callable(progressCallback):
+                    progressCallback({"current": offset, "total": size})
+
+        if offset != size:
+            self.delete("file/upload/" + uploadId)
+            raise gcl.IncorrectUploadLengthError(
+                "Expected upload to be %d bytes, but received %d." % (size, offset),
+                upload=uploadObj,
+            )
+
+        return uploadObj
+
 
 def _harmonize_girder_dandiset_to_dandi_api(rec):
     """
     Compare API (on a released version):
 
-{'count': 1,
- 'created': '2020-07-21T22:22:15.396171Z',
- 'dandiset': {'created': '2020-07-21T22:22:14.732729Z',
-              'identifier': '000027',
-              'updated': '2020-07-21T22:22:14.732762Z'},
- 'metadata': {'dandiset': {...}},
- 'updated': '2020-07-21T22:22:15.396295Z',
- 'version': '0.200721.2222'}
+    {'count': 1,
+     'created': '2020-07-21T22:22:15.396171Z',
+     'dandiset': {'created': '2020-07-21T22:22:14.732729Z',
+                  'identifier': '000027',
+                  'updated': '2020-07-21T22:22:14.732762Z'},
+     'metadata': {'dandiset': {...}},
+     'updated': '2020-07-21T22:22:15.396295Z',
+     'version': '0.200721.2222'}
 
     to Girder (on drafts):
 
-{'attrs': {'ctime': '2020-07-08T21:54:42.543000+00:00',
-           'mtime': '2020-07-21T22:02:34.918000+00:00',
-           'size': 0},
- 'id': '5f0640a2ab90ac46c4561e4f',
- 'metadata': {'dandiset': {...}},
- 'name': '000027',
- 'path': '000027',
- 'type': 'dandiset'}
+    {'attrs': {'ctime': '2020-07-08T21:54:42.543000+00:00',
+               'mtime': '2020-07-21T22:02:34.918000+00:00',
+               'size': 0},
+     'id': '5f0640a2ab90ac46c4561e4f',
+     'metadata': {'dandiset': {...}},
+     'name': '000027',
+     'path': '000027',
+     'type': 'dandiset'}
 
-So we will place some girder specific ones under 'girder' and populate 'dandiset', e.g.
-(there is absent clarify of what date times API returns:
-https://github.com/dandi/dandi-publish/issues/107
-so we will assume that my take was more or less correct and then we would have them
-correspond in case of a draft, as it is served by girder ATM:
-
-{# 'count': 1,  # no count
- 'created': '2020-07-21T22:22:15.396171Z',  # attrs.ctime
- 'dandiset': {'created': '2020-07-08T21:54:42.543000+00:00',  # attrs.ctime
-              'identifier': '000027',  # name
-              'updated': '2020-07-21T22:02:34.918000+00:00' },  # attrs.mtime
- 'metadata': {'dandiset': {...}},
- 'updated': '2020-07-21T22:02:34.918000+00:00'}  # attrs.mtime
+    So we will place some girder specific ones under 'girder' and populate 'dandiset', e.g.
+    (there is absent clarify of what date times API returns:
+    https://github.com/dandi/dandi-publish/issues/107
+    so we will assume that my take was more or less correct and then we would have them
+    correspond in case of a draft, as it is served by girder ATM:
 
+    {# 'count': 1,  # no count
+     'created': '2020-07-21T22:22:15.396171Z',  # attrs.ctime
+     'dandiset': {'created': '2020-07-08T21:54:42.543000+00:00',  # attrs.ctime
+                  'identifier': '000027',  # name
+                  'updated': '2020-07-21T22:02:34.918000+00:00' },  # attrs.mtime
+     'metadata': {'dandiset': {...}},
+     'updated': '2020-07-21T22:02:34.918000+00:00'}  # attrs.mtime
 
     Parameters
     ----------
@@ -629,53 +692,52 @@ def _get_file_mtime(attrs):
 
 def _harmonize_girder_asset_to_dandi_api(rec):
     """
-
     girder rec:
 
-*(Pdb) pprint(_a[0])
-{'attrs': {'ctime': '2020-07-21T22:00:36.362000+00:00',
-           'mtime': '2020-07-21T17:31:55.283394-04:00',
-           'size': 18792},
- 'id': '5f176584f63d62e1dbd06946',
- 'metadata': {... identical at this level
-              'uploaded_by': 'dandi 0.5.0+12.gd4ef762.dirty',
-              'uploaded_datetime': '2020-07-21T18:00:36.703727-04:00',
-              'uploaded_mtime': '2020-07-21T17:31:55.283394-04:00',
-              'uploaded_size': 18792},
- 'name': 'sub-RAT123.nwb',
- 'path': '000027/sub-RAT123/sub-RAT123.nwb',
- 'type': 'file'}
+    *(Pdb) pprint(_a[0])
+    {'attrs': {'ctime': '2020-07-21T22:00:36.362000+00:00',
+               'mtime': '2020-07-21T17:31:55.283394-04:00',
+               'size': 18792},
+     'id': '5f176584f63d62e1dbd06946',
+     'metadata': {... identical at this level
+                  'uploaded_by': 'dandi 0.5.0+12.gd4ef762.dirty',
+                  'uploaded_datetime': '2020-07-21T18:00:36.703727-04:00',
+                  'uploaded_mtime': '2020-07-21T17:31:55.283394-04:00',
+                  'uploaded_size': 18792},
+     'name': 'sub-RAT123.nwb',
+     'path': '000027/sub-RAT123/sub-RAT123.nwb',
+     'type': 'file'}
 
     and API (lacking clear "modified" so needs tuning too):
 
-    {
-      "version": {
-        "dandiset": {
-          "identifier": "000027",
-          "created": "2020-07-21T22:22:14.732729Z",
-          "updated": "2020-07-21T22:22:14.732762Z"
-        },
-        "version": "0.200721.2222",
-        "created": "2020-07-21T22:22:15.396171Z",
-        "updated": "2020-07-21T22:22:15.396295Z",
-        "count": 1
-      },
-      "uuid": "bca53c42-7fc2-41b6-b836-5ed102ba8447",
-      "path": "/sub-RAT123/sub-RAT123.nwb",
-      "size": 18792,
-      "sha256": "1a765509384ea96b7b12136353d9c5b94f23d764ad0431e049197f7875eb352c",
-      "created": "2020-07-21T22:22:16.882594Z",
-      "updated": "2020-07-21T22:22:16.882641Z",
-      "metadata": {
-...
-        "sha256": "1a765509384ea96b7b12136353d9c5b94f23d764ad0431e049197f7875eb352c",
-...
-        "uploaded_size": 18792,
-        "uploaded_mtime": "2020-07-21T17:31:55.283394-04:00",
-        "uploaded_datetime": "2020-07-21T18:00:36.703727-04:00",
-...
-      }
-    }
+        {
+          "version": {
+            "dandiset": {
+              "identifier": "000027",
+              "created": "2020-07-21T22:22:14.732729Z",
+              "updated": "2020-07-21T22:22:14.732762Z"
+            },
+            "version": "0.200721.2222",
+            "created": "2020-07-21T22:22:15.396171Z",
+            "updated": "2020-07-21T22:22:15.396295Z",
+            "count": 1
+          },
+          "uuid": "bca53c42-7fc2-41b6-b836-5ed102ba8447",
+          "path": "/sub-RAT123/sub-RAT123.nwb",
+          "size": 18792,
+          "sha256": "1a765509384ea96b7b12136353d9c5b94f23d764ad0431e049197f7875eb352c",
+          "created": "2020-07-21T22:22:16.882594Z",
+          "updated": "2020-07-21T22:22:16.882641Z",
+          "metadata": {
+    ...
+            "sha256": "1a765509384ea96b7b12136353d9c5b94f23d764ad0431e049197f7875eb352c",
+    ...
+            "uploaded_size": 18792,
+            "uploaded_mtime": "2020-07-21T17:31:55.283394-04:00",
+            "uploaded_datetime": "2020-07-21T18:00:36.703727-04:00",
+    ...
+          }
+        }
 
 
     Parameters
@@ -684,7 +746,6 @@ def _harmonize_girder_asset_to_dandi_api(rec):
 
     Returns
     -------
-
     """
     rec = rec.copy()  # we will modify in place
 
@@ -912,7 +973,9 @@ def keyring_lookup(service_name, username):
             lgr.info("EncryptedKeyring file exists; using as keyring backend")
             return (kb, kb.get_password(service_name, username))
         lgr.info("EncryptedKeyring file does not exist")
-        if askyesno("Would you like to establish an encrypted keyring?", default=True):
+        if click.confirm(
+            "Would you like to establish an encrypted keyring?", default=True
+        ):
             keyring_cfg = Path(keyringrc_file())
             if keyring_cfg.exists():
                 lgr.info("%s exists; refusing to overwrite", keyring_cfg)
