@@ -4,29 +4,26 @@ import io
 import itertools
 import os
 import os.path as op
+from pathlib import Path
 import platform
 import re
 import shutil
 import subprocess
 import sys
+from time import sleep
 
-from pathlib import Path
-
+import dateutil.parser
 import requests
 import ruamel.yaml
 from semantic_version import Version
 
-from . import __version__
 from .consts import dandi_instance, known_instances, known_instances_rev
 from .exceptions import BadCliVersionError, CliVersionTooOldError
 
-import dateutil.parser
-
 #
 # Additional handlers
 #
-from . import get_logger
-
+from . import __version__, get_logger
 
 lgr = get_logger()
 
@@ -45,6 +42,13 @@ on_msys_tainted_paths = (
     and os.environ.get("MSYSTEM", "")[:4] in ("MSYS", "MING")
 )
 
+USER_AGENT = "dandi/{} requests/{} {}/{}".format(
+    __version__,
+    requests.__version__,
+    platform.python_implementation(),
+    platform.python_version(),
+)
+
 
 def is_interactive():
     """Return True if all in/outs are tty"""
@@ -56,8 +60,8 @@ def is_interactive():
 def setup_exceptionhook(ipython=False):
     """Overloads default sys.excepthook with our exceptionhook handler.
 
-       If interactive, our exceptionhook handler will invoke
-       pdb.post_mortem; if not interactive, then invokes default handler.
+    If interactive, our exceptionhook handler will invoke
+    pdb.post_mortem; if not interactive, then invokes default handler.
     """
 
     def _pdb_excepthook(type, value, tb):
@@ -148,19 +152,13 @@ def ensure_strtime(t, isoformat=True):
     raise TypeError(f"Do not know how to convert {t_orig!r} to string datetime")
 
 
-def fromisoformat(t, impl=None):
-    if impl is None:
-        # datetime parser "does not support parsing arbitrary ISO 8601 strings"
-        # https://docs.python.org/3/library/datetime.html
-        # In particular it does not parse time zone suffix which was recently
-        # introduced into datetime's provided by API
-        impl = "dateutil"
-    if impl == "datetime":
-        return datetime.datetime.fromisoformat(t)
-    elif impl == "dateutil":
-        return dateutil.parser.isoparse(t)
-    else:
-        raise ValueError(impl)
+def fromisoformat(t):
+    # datetime.fromisoformat "does not support parsing arbitrary ISO 8601
+    # strings" <https://docs.python.org/3/library/datetime.html>.  In
+    # particular, it does not parse the time zone suffixes recently
+    # introduced into timestamps provided by the API.  Hence, we need to use
+    # dateutil instead.
+    return dateutil.parser.isoparse(t)
 
 
 def ensure_datetime(t, strip_tzinfo=False, tz=None):
@@ -381,15 +379,12 @@ def copy_file(src, dst):
 
 
 def move_file(src, dst):
-    """Move file from src to dst
-
-    """
+    """Move file from src to dst"""
     return shutil.move(src, dst)
 
 
 def find_dandi_files(paths):
-    """Adapter to find_files to find files of interest to dandi project
-    """
+    """Adapter to find_files to find files of interest to dandi project"""
     sep = re.escape(os.sep)
     yield from find_files(rf"((^|{sep})dandiset\.yaml|\.nwb)\Z", paths)
 
@@ -559,8 +554,7 @@ def auto_repr(cls):
 
 
 def Parallel(**kwargs):  # TODO: disable lint complaint
-    """Adapter for joblib.Parallel so we could if desired, centralize control
-    """
+    """Adapter for joblib.Parallel so we could if desired, centralize control"""
     # ATM just a straight invocation
     import joblib
 
@@ -568,8 +562,7 @@ def Parallel(**kwargs):  # TODO: disable lint complaint
 
 
 def delayed(*args, **kwargs):
-    """Adapter for joblib.delayed so we could if desired, centralize control
-    """
+    """Adapter for joblib.delayed so we could if desired, centralize control"""
     # ATM just a straight invocation
     import joblib
 
@@ -617,8 +610,69 @@ def get_instance(dandi_instance_id):
     if our_version in bad_versions:
         raise BadCliVersionError(our_version, minversion, bad_versions)
     return dandi_instance(
+        metadata_version=0,
         girder=server_info["services"]["girder"]["url"],
         gui=server_info["services"]["webui"]["url"],
         redirector=redirector_url,
         api=server_info["services"]["api"]["url"],
     )
+
+
+TITLE_CASE_LOWER = {
+    "a",
+    "an",
+    "and",
+    "as",
+    "but",
+    "by",
+    "for",
+    "in",
+    "nor",
+    "of",
+    "on",
+    "or",
+    "the",
+    "to",
+    "with",
+}
+
+
+def name2title(name):
+    # For use in autopopulating the titles of model schema fields
+    words = []
+    for w in split_camel_case(name):
+        w = w.lower()
+        if w == "id" or w == "url":
+            w = w.upper()
+        elif not words or w not in TITLE_CASE_LOWER:
+            w = w.capitalize()
+        words.append(w)
+    return " ".join(words)
+
+
+def split_camel_case(s):
+    last_start = 0
+    # Don't split apart "ID":
+    for m in re.finditer(r"(?<=I)[A-CE-Z]|(?<=[^I])[A-Z]", s):
+        yield s[last_start : m.start()]
+        last_start = m.start()
+    if last_start < len(s):
+        yield s[last_start:]
+
+
+def try_multiple(ntrials, exception, base, f, *args, **kwargs):
+    """
+    Call ``f`` multiple times until it succeeds, with exponentially increasing
+    delay between calls
+    """
+    for trial in range(1, ntrials + 1):
+        try:
+            return f(*args, **kwargs)
+        except exception as exc:
+            if trial == ntrials:
+                raise  # just reraise on the last trial
+            t = base ** trial
+            lgr.warning(
+                "Caught %s on trial #%d. Sleeping %f and retrying", exc, trial, t
+            )
+            sleep(t)
